// Four numbers that define the transform, i hat and j hat
const i_x = 1;
const i_y = 0.5;
const j_x = -1;
const j_y = 0.5;

// Symmetric 2D array to store instances of TileClass
const tile_arr_size = 21;
let tileArr = new Array(tile_arr_size).fill(null).map(() => new Array(tile_arr_size).fill(null));

// Sprite size
const w = 64;
const h = 64;

// what the current cursor calculation is based off
let maxWinWidth = 1920 ;
let maxWinHeight = 1080;


function preload() {
    tile = loadImage("TileDefault.png")
    tile2 = loadImage("TileHighlight.png");
    tile3 = loadImage("TileThree.png");
    tile4 = loadImage("TileRoundAbout.png");
}


function setup() {
    createCanvas(maxWinWidth, maxWinHeight);
    imageMode(CENTER);
    tile.resize(64, 64)
    tile2.resize(64, 64)
    tile3.resize(64, 64)
    // roundabout
    tile4.resize(256, 256)
    for (let i = 0; i < tile_arr_size; i++) {
        for (let j = 0; j < tile_arr_size; j++) {
            let tsc = toScreenCoords({
                x: i,
                y: j
            });
            
            tileArr[i][j] = new TileClass(i, j, tsc.x, tsc.y, tile);
            tileArr[i][j].display(tileArr[i][j].sprite);
        }
    }
}

// Current Sprite Selection from User -- might not be needed
let currUserSprite;

function draw() {
    currUserSprite = tile3;
    background('#588dbe');
    translate(width / 2, height / 3)


    // uncomment to debug/find offset calculation:
    // console.log(screenToGridCoords({x: mouseX, y: mouseY}));
    let tgc = screenToGridCoords({x: mouseX, y: mouseY});

    for (let i = 0; i < tile_arr_size; i++) {
        for (let j = 0; j < tile_arr_size; j++) {
            tileArr[i][j].display(tileArr[i][j].sprite);

            // hover highlighting 
            if (tgc.x == i && tgc.y == j && tileArr[i][j].sprite == tile) {
              tileArr[i][j].display(tile2);
            } else {
              tileArr[i][j].display(tileArr[i][j].sprite);
            }

        }
    }

    // place ment of roundabout in center -ish 
    image(tile4, -35, 292)
}

function toScreenCoords(tile) {
    return {
        x: tile.x * i_x * 0.5 * w + tile.y * j_x * 0.5 * w,
        y: tile.x * i_y * 0.5 * h + tile.y * j_y * 0.5 * h,
    }
}

function invertMatrix(a, b, c, d) {
    const det = (1 / (a * d - b * c));

    return {
        a: det * d,
        b: det * -b,
        c: det * -c,
        d: det * a,
    }
}

function toGridCoords(screen) {
    const a = i_x * 0.5 * w;
    const b = j_x * 0.5 * w;
    const c = i_y * 0.5 * h;
    const d = j_y * 0.5 * h;

    const inv = invertMatrix(a, b, c, d);

    return {
        // offset mouse cursor to align with block
        x: screen.x * inv.a + screen.y * inv.b - 25.75,
        y: screen.x * inv.c + screen.y * inv.d + 4.15,
    }
}

// Helper function to convert screen coordinates to grid coordinates
function screenToGridCoords(screen) {
    let tgc = toGridCoords(screen);
    return {
        x: Math.floor(tgc.x),
        y: Math.floor(tgc.y)
    };
}

// Helper function to check if grid coordinates are within bounds
function isValidGridCoords(tgc_x, tgc_y) {
    return tgc_x >= 0 && tgc_x < tile_arr_size && tgc_y >= 0 && tgc_y < tile_arr_size;
}

// Helper function to update tile sprite
function updateTileSprite(tgc_x, tgc_y, sprite) {
    tileArr[tgc_x][tgc_y].sprite = sprite;
}

function mouseClicked() {
    let {
        x: tgc_x,
        y: tgc_y
    } = screenToGridCoords({
        x: mouseX,
        y: mouseY
    });
    if (!isValidGridCoords(tgc_x, tgc_y)) {
        return;
    }
    updateTileSprite(tgc_x, tgc_y, currUserSprite);
}

function rotateMatrixClockwise(matrix) {
    let rotatedMatrix = new Array(tile_arr_size).fill(null).map(() => new Array(tile_arr_size).fill(null));

    for (let i = 0; i < tile_arr_size; i++) {
        for (let j = 0; j < tile_arr_size; j++) {
            let newX = j;
            let newY = tile_arr_size - 1 - i;
            let tile = matrix[i][j];
            let screenCoord = toScreenCoords({
                x: newX,
                y: newY
            });
            rotatedMatrix[newX][newY] = new TileClass(newX, newY, screenCoord.x, screenCoord.y, tile.sprite);
        }
    }
    return rotatedMatrix;
}

function keyPressed() {
    if (key === 'r') {
        tileArr = rotateMatrixClockwise(tileArr);
    }
}
